#!/bin/sh
# Marten Vijn 2010 
# BSD license 
# full text: https://projects.math.leidenuniv.nl/trac/howto/wiki/mileiden_license
#

. ./config.txt


## NOTE on the function DIE !!!
# The funtion DIE is sourced from config.txt
# It's used multiple times so leave it there
# until the moment we have a library file for functions


## set env
BUILD_CMD=$1

if  [ $BUILDVERSION = "SVN" ]
then
	BUILDVERSION=`svn info | grep Revision | cut -d ' ' -f2`
fi 

DIR=${ROOT}/${VER}
PXE_DIR=${PXE_ROOT}/${VER}

## check variables 

VARS="DIR ROOT PXE_ROOT VER BUILDVERSION NAMESERVER FILE_DIR MIRROR NAT_ENABLE NAT_INTERFACE MANIFEST_REMOVE CASPER_KEEP"

for VAR in $VARS
do
	echo $VAR
	if [ "$(eval echo \$${VAR})x" = "x" ]
	then
		echo  "The variable $VAR is empty and can cause damage "
		echo "exiting"
		exit 1
	fi

done

# Check Dirs

mkdir -p $DIR || DIE
mkdir -p $PXE_DIR || DIE

# Functions 
NET_CHECK(){
	if [ "$NET_ENABLE" = "YES" ]
	then
		modprobe iptable_nat
		iptables -t nat -A POSTROUTING -o $NAT_INTERFACE -j MASQUERADE
		echo 1 > /proc/sys/net/ipv4/ip_forward
	fi
	if [ `ping -c 1 example.com | grep -c rtt` = "0" ]
	then
		echo "no network .... fix that first...!"
		exit 1
	fi
	}
CLEAN_BUILDDIR(){
	if [ -d $DIR ]
	then
		for DEV in sys proc dev
		do
			umount ${DIR}/${DEV}	
		done
		rm -rf ${DIR}
	fi	

	}

CLEAN_INSTALLDIR(){
	MSG="cleanup install dir" && echo $MSG 
	if [ -d ${PXE_DIR}_old ] ; then
		rm -rf ${PXE_DIR}_old || DIE
	fi
	if [ -d ${PXE_DIR} ] ; then
		mv ${PXE_DIR} ${PXE_DIR}_old || DIE
	fi
	mkdir -p ${PXE_DIR}/casper 
	}

### generic karmic 
BUILD_BASE(){

	# build base dist
	MSG="install base install with debootstrap" && echo $MSG 
	mkdir -p $DIR 
	debootstrap --arch=i386 karmic $DIR $MIRROR || DIE
	}

### stage2  preparing the tree for usage installing desktop and utils 
# and optionally build a kernel

BUILD_KERNEL_PKG(){
	if [ "$BUILD_KERNEL" = "YES" ] || [ "$BUILD_CMD" = "BUILD_KERNEL" ]
	then
		for DEB in `ls ${DIR}/usr/src/*.deb`
		do  
			rm ${DIR}/usr/src/$DEB || DIE
		done
		
		cp kernel.config ${DIR} || echo 'CONFIG_KERNEL="YES"' >> ${DIR}/config.txt 
		
		echo "BUILDVERSION=\"${BUILDVERSION}\"" >> ${DIR}/config.txt
 
		cat > ${DIR}/build-kernel.sh << EOF
#!/bin/sh	
. /config.txt
export LANG=en_US.UTF-8  || DIE
export HOME=/root || DIE
export LC_ALL=C || DIE
apt-get install --yes --force-yes kernel-package $KERNEL_SRC_PKG libncurses5-dev ncurses-base || DIE
tar -xjvf  /usr/src/${KERNEL_SRC_PKG}.tar.bz2 -C /usr/src/ || DIE
ln -s  /usr/src/${KERNEL_SRC_PKG} /usr/src/linux || DIE
cd /usr/src/linux || DIE
cp /kernel.config  .config || DIE
if [ "x${CONFIG_KERNEL}" = "xYES" ] || DIE
then
	make menuconfig  || DIE
fi 
fakeroot make-kpkg clean || DIE
fakeroot make-kpkg --initrd --append-to-version=-${BUILDVERSION} kernel-image kernel-headers || DIE
apt-get remove --yes kernel-package $KERNEL_SRC_PKG || DIE
exit 0
EOF

		chroot $DIR sh -x /build-kernel.sh || DIE
		if [ "$CONFIG_KERNEL" = "YES" ]
        	then
			cp ${DIR}/usr/src/${KERNEL_SRC_PKG}/.config kernel.config.new 
		fi

### This needs testing 
#		for FILE in "build-kernel.sh kernel.config"
#		do
#			rm ${DIR}/${FILE} || DIE
#		done
		for DEB in `ls ${DIR}/usr/src/*.deb` 
		do  
			cp $DEB ${ROOT} || DIE
			rm $DEB || DIE
		done
	fi
	}

STAGE2(){
	if [ ! -f ${DIR}/etc/hosts ]
	then
		echo "127.0.0.1 localhost" > ${DIR}/etc/hosts || DIE
	fi

	## prepare chroot 
	MSG="prepare chroot for stage2" && echo $MSG 
	echo "nameserver $NAMESERVER" > ${DIR}/etc/resolv.conf
	B_FILES="config.txt dpkg-set-selections.txt preseed.txt"

	for FILE in $B_FILES
	do
		cp ${ROOT}/$FILE ${DIR}/ || DIE
	done
	
	## only builds KERNEL package if configured or absent
	
	KERNEL_PKG=`ls | grep ^linux-image-2.6 | tail -n 1`
	if [  "x$KERNEL_PKG" = "x" ]
	then
		BUILD_KERNEL="YES"
		BUILD_KERNEL_PKG
	else
		BUILD_KERNEL_PKG
	fi	

	KERNEL_HDR=`ls | grep ^linux-header-2.6 | tail -n 1`
	 if [ "x$KERNEL_PKG" = "x" ]
        then
                BUILD_KERNEL="YES"
                BUILD_KERNEL_PKG
                  
        else
                BUILD_KERNEL_PKG
        fi

	MSG="copying kernel packages" && echo $MSG 
	KERNEL_PKG=`ls linux-image*deb | tail -n 1`
	KERNEL_HDR=`ls linux-head*deb | tail -n 1`
	
	cp -v $KERNEL_PKG ${DIR}/  || DIE
	cp -v $KERNEL_HDR ${DIR}/  || DIE

  	echo "do_initrd = Yes"   > ${DIR}/etc/kernel-img.conf  || DIE
  	echo "do_symlinks = Yes"   >> ${DIR}/etc/kernel-img.conf || DIE 
	
	echo "deb $MIRROR karmic main restricted universe multiverse" > ${DIR}/etc/apt/sources.list || DIE
	### Stage 2 install packages in chroot
	
	cat > ${DIR}/stage2.sh << EOF 
#!/bin/sh
# 
export PATH="/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin"

. /config.txt 

mount -t proc none /proc 
mount -t sysfs none /sys 
mount -t devpts none /dev/pts 
export HOME=/root || DIE
export LC_ALL=C || DIE

apt-get update || DIE
cat /preseed.txt | debconf-set-selections  || DIE


## debconf timezone settings don't work... 
# need some hacking to get to work..
# dpkg-reconfigure -f noninteractive tzdata
# above should work, but does not ?

if [ "x${TIME_ZONE}" = "x" ]
then
	echo "No time zone is set"
else	
	echo "Setting time zone to ${TIME_ZONE}" 	  
	ln -sf /usr/share/zoneinfo/${TIME_ZONE} /etc/localtime || DIE
fi
### temp fix for boot issue, disable later
dpkg-divert --local --rename --add /sbin/initctl || DIE
ln -fs /bin/true /sbin/initctl || DIE

## install kernel + headers
dpkg -i /linux*deb || DIE

cat /dpkg-set-selections.txt | dpkg --set-selections  || DIE
apt-get -y dselect-upgrade || DIE

# using debconf settings
apt-get update || DIE
apt-get upgrade || DIE

## delete these packages
DEL_PKGS="jockey-gtk"
for PKG in $DEL_PKGS
do
	apt-get remove --yes $PKG || DIE
done

## clean up workaround, remove later 
rm /sbin/initctl || DIE
dpkg-divert --rename --remove /sbin/initctl ||  DIE

## generic cleanup
apt-get -y autoremove || DIE
apt-get clean || DIE
updatedb || DIE
rm -rf /tmp/* || DIE
umount -f /proc || DIE
umount -f /sys  || DIE  
umount -f /dev/pts || DIE

exit 0
EOF
	chmod 755 ${DIR}/stage2.sh || DIE
	chroot $DIR sh /stage2.sh || DIE

	B_FILES="$B_FILES stage2.sh"
	for FILE in $B_FILES
	do
		if [ -f ${DIR}/${FILE} ]
		then
			rm ${DIR}/${FILE} || DIE
		fi
	done
	
	# cleanup install files 
	rm -v ${DIR}/linux*deb || DIE
	rm -v ${DIR}/etc/resolv.conf || DIE

	echo $BUILDVERSION > ${DIR}/etc/BUILD_VERSION || DIE
	}

### Copy my files from FILE_DIR
MY_FILES(){
	MSG="installing my file from ${FILE_DIR}" && echo $MSG 
	
	# some checks 
	if [ "$FILE_DIR" = "NONE" ] || [ "x$FILE_DIR" = "x" ] 
	then
		echo "no files to copy"
	else
		if [ ! -d $FILE_DIR ] 
		then
			echo "$FILE_DIR does not exist"
			exit 1
		fi	
		
		(
		cd $FILE_DIR || DIE
		tar -cvf - --exclude=.svn . | tar -xpf - -C ${DIR}/ || DIE
		)	
	fi
	
	if [ -f ${DIR}/etc/rc.local ]
	then
		MSG="Exec permissions for rc.local" &&  echo $MSG 
		chmod 755 ${DIR}/etc/rc.local || DIE 
	fi
	}


CASPER(){
	MSG="setting up casper" && echo "$MSG" 
	if [ -d /tmp/casper_tmp ]
		then
		rm -rf /tmp/casper_tmp
	fi
	mkdir /tmp/casper_tmp
	mv ${DIR}/initrd.img ${DIR}/initrd.img_orig || DIE

	cd /tmp/casper_tmp
	7z e -so ${DIR}/initrd.img_orig | cpio -id || DIE
	
	for ITEM in `ls /tmp/casper_tmp/scripts/casper-bottom/`
	do
		if [ `echo $CASPER_KEEP | grep -c  $ITEM` = "1" ]
		then 
			echo "Casper keeps $ITEM"	
		else 
			echo "Casper removes $ITEM"
			rm /tmp/casper_tmp/scripts/casper-bottom/$ITEM || DIE
		
		fi
	done

	find . | cpio -o -H newc | gzip > ${DIR}/initrd.img || DIE
	cd ${ROOT}	
	}

# acroread

ACROREAD(){
	if [ "$ACROREAD_INSTALL" = "YES" ]
	then
		MSG="getting acroread" && echo "$MSG"
		if [ ! -f $ACROREAD_DEB ]
		then
			wget ${ACROREAD_URL}/${ACROREAD_DEB} || DIE
		fi
	
		cp $ACROREAD_DEB ${DIR}/${ACROREAD_DEB}
		
		MSG="installing acroread ${ACROREAD_DEB}" && echo "$MSG"
		chroot ${DIR} dpkg -i /${ACROREAD_DEB} || DIE
		rm ${DIR}/${ACROREAD_DEB} || DIE
	fi
	} 


### build squashfs

BUILD_SQUASH(){
	MSG="Building squash image and setting PXE environment" && echo $MSG 
	### pxe env
	mkdir -p ${PXE_DIR}
	cp ${DIR}/vmlinuz ${PXE_DIR}/vmlinuz || DIE
	cp ${DIR}/initrd.img ${PXE_DIR}/initrd.img || DIE
	
	chroot $DIR dpkg-query -W --showformat='${Package} ${Version}\n' > ${PXE_DIR}/casper/filesystem.manifest || DIE

	cp ${PXE_DIR}/casper/filesystem.manifest ${PXE_DIR}/casper/filesystem.manifest-desktop | DIE 

	for i in $MANIFEST_REMOVE 
	do
        	sed -i "/${i}/d" ${PXE_DIR}/casper/filesystem.manifest-desktop  || DIE
	done

	mksquashfs ${DIR} ${PXE_DIR}/casper/filesystem.squashfs || DIE
	}


case $BUILD_CMD in
	BUILD_KERNEL)
		BUILD_KERNEL_PKG
		;;
	DO_FAST)
		CLEAN_INSTALLDIR 
		STAGE2
		MY_FILES		
		CASPER
		BUILD_SQUASH
		;;	
	
	*) 
		CLEAN_BUILDDIR
		CLEAN_INSTALLDIR
		NET_CHECK
		BUILD_BASE
		STAGE2
		ACROREAD
		MY_FILES		
		CASPER
		BUILD_SQUASH
	;;
esac



